package com.kohabi.hnbrowser;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.LoaderManager.LoaderCallbacks;
import android.support.v4.content.Loader;
import android.os.Bundle;
import android.view.View;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ListView;

import java.util.List;

public class NewsFeedActivity extends FragmentActivity
	implements LoaderCallbacks<Either<Exception, List<Article>>>,
			   OnItemClickListener, OnItemLongClickListener {
	private ListView listView;
	private ArticleAdapter adapter;
	private ProgressDialog dialog;

	@Override protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.news_feed_activity);
		adapter = new ArticleAdapter(getLayoutInflater());
		listView = (ListView) findViewById(R.id.list);
		listView.setAdapter(adapter);
		listView.setOnItemClickListener(this);
		listView.setOnItemLongClickListener(this);
		refresh();
	}

	@Override public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.news_feed_activity, menu);
		return true;
	}

	@Override public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
			case R.id.action_refresh:
				refresh();
				return true;
			default:
				return super.onOptionsItemSelected(item);
		}
	}

	@Override public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		Article article = adapter.getItem(position);
		Intent intent = new Intent(Intent.ACTION_VIEW);
		intent.setData(Uri.parse(article.url));
		startActivity(intent);
	}

	@Override public boolean onItemLongClick(AdapterView<?> parent, View view,
											 int position, long id) {
		Article article = adapter.getItem(position);
		Intent intent = new Intent(this, CommentsActivity.class);
		intent.putExtra(CommentsActivity.COMMENTS_URI, article.commentsUri);
		startActivity(intent);
		return true;
	}

	@Override public Loader<Either<Exception, List<Article>>> onCreateLoader(int id, Bundle args) {
		return new ArticleLoader(this);
	}

	@Override public void onLoaderReset(Loader<Either<Exception, List<Article>>> loader) {}

	@Override public void onLoadFinished(Loader<Either<Exception, List<Article>>> loader,
							   Either<Exception, List<Article>> data) {
		dialog.dismiss();
		if (data.isLeft()) {
			Exception e = data.getLeft();
			(new AlertDialog.Builder(this))
					.setTitle("Error")
					.setMessage(e.toString())
					.setCancelable(true)
					.setPositiveButton("Okay", new DialogInterface.OnClickListener() {
						public void onClick(DialogInterface d, int id) {
							d.cancel();
						}
					}).create().show();
		} else {
			List<Article> articles = data.getRight();
			for (Article article : articles) {
				adapter.add(article);
			}
		}
	}

	private void refresh() {
		adapter.clear();
		dialog = new ProgressDialog(this);
		dialog.setTitle("Loading..");
		dialog.setMessage("Please wait.");
		dialog.setCancelable(false);
		dialog.setIndeterminate(true);
		dialog.show();
		getSupportLoaderManager().restartLoader(0, null, this).forceLoad();
	}
}
